/*
 Jed Margolin  11/23/2021
 Read the HTU21D and send the temperature and humidity to the display
 Uses I2C Interface for the HTU21D
 Displays the Raw Temperature, Temperature in C, Temperature in F, and Humidity on the LCD

 Use MSP430G2553, MSP430G2402 or other with 8K Program Memory

 Compiler Optimization must be set to 0-Register Optimization only

 P1.0  out  for testing
 P1.1  in   not used
 P1.2  in   not used
 P1.3  in   not used
 P1.4  in   not used
 P1.5  out  htu21d SCL
 P1.6  in   not used
 P1.7  out  htu21d SDA

 P2.0  Out  LCD - D4
 P2.1  out  LCD - D5
 P2.2  out  LCD - D6
 P2.3  out  LCD - D7
 P2.4  out  LCD - /E
 P2.5  out  LCD - RS
 P2.6  in   not used (XIN)
 P2.7  in   not used (XOUT)
*/

//==================================================================
#define MSPTYPE 0   // 1 = MSP430G2553, 0 = MSP430G2402 or other
#define TESTLOOP 0

#include <msp430.h>
#include "main.h"
#include <htu21.h>

int main(void)
{
    WDTCTL = WDTPW | WDTHOLD;   // stop watchdog timer

// No crystal. Use the factory default values
    DCOCTL = CALDCO_1MHZ;      // factory calibration vqlues
    BCSCTL1 = CALBC1_1MHZ;
    BCSCTL2 = 0x06;            // DIVS (Divider for SMCLK) /8

// Port 1 is all digital I/O
    P1SEL  &= ~(BIT0|BIT1|BIT2|BIT3|BIT4|BIT5|BIT6|BIT7);
    P1SEL2 &= ~(BIT0|BIT1|BIT2|BIT3|BIT4|BIT5|BIT6|BIT7);

// Port 1 - inputs
    P1REN |= (BIT1|BIT2|BIT3|BIT4|BIT6);    // pullup/pulldown enabled
    P1OUT |= (BIT1|BIT2|BIT3|BIT4|BIT6);    //
    P1DIR &= ~(BIT1|BIT2|BIT3|BIT4|BIT6);   // In

// Port 1 - outputs
    P1DIR |= (BIT0|BIT5|BIT7);
    P1OUT &= (BIT0|BIT5|BIT7);

// Port 2 is all digital I/O
// Since we are not using a crystal XIN is P2.6 and XOUT is P2.7
    P2SEL  &= ~(BIT0|BIT1|BIT2|BIT3|BIT4|BIT5|BIT6|BIT7);
    P2SEL2 &= ~(BIT0|BIT1|BIT2|BIT3|BIT4|BIT5|BIT6|BIT7);

// Port 2 - outputs for LCD
    P2DIR |= (BIT0|BIT1|BIT2|BIT3|BIT4|BIT5);     //Out
    P2OUT &= ~(BIT0|BIT1|BIT2|BIT3|BIT4|BIT5);

// Port 2 - inputs
    P2REN |= (BIT6|BIT7);    // pullup/pulldown enabled
    P2OUT |= (BIT6|BIT7);    //
    P2DIR &= ~(BIT6|BIT7);   // In

#if MSPTYPE
// Port 3 - inputs (the 2553 die has 3 ports)
    P3SEL  &= ~(BIT0|BIT1|BIT2|BIT3|BIT4|BIT5|BIT6|BIT7);  // In
    P3SEL2 &= ~(BIT0|BIT1|BIT2|BIT3|BIT4|BIT5|BIT6|BIT7);
    P3REN |= (BIT0|BIT1|BIT2|BIT3|BIT4|BIT5|BIT6|BIT7);    // pullup/pulldown enabled
    P3OUT |= (BIT0|BIT1|BIT2|BIT3|BIT4|BIT5|BIT6|BIT7);    //
    P3DIR &= ~(BIT0|BIT1|BIT2|BIT3|BIT4|BIT5|BIT6|BIT7);   // In
#endif

    lcd_init();     // initialize lcd

    htu_init();     // soft reset
    do_register();  // make sure the User Register is set properly

// reading the temperature and the humidity takes 104ms with 0-Register Optimizations
    temp_C = get_htu_temperature();   // total time to read and convert the temperature is 104ms
    F = (temp_C * 9 / 5) +320;    // Fahrenheit * 10
    Humidity = get_htu_humidity();

    waitms(2000);
    lcd_display();

    asm(" nop");    // for debugger

#if TESTLOOP
// Loop for testing
   while (1)
   {

      P1OUT ^= BIT0;

      temp_C = get_htu_temperature();   // total time to read and convert the temperature and do humidity is 104ms
      F = (temp_C * 9 / 5) +320;    // Fahrenheit * 10
      Humidity = get_htu_humidity();

      asm(" nop");    // for debugger

   }  // end while test loop
#endif

// Set up timer Interrupts - ms
    CCTL0 = CCIE;                    // CCR0 interrupt enabled
    CCR0 = 31600;                    // 1 second interrupts (15800 = 500ms)
    TACTL = TASSEL_2 | ID_2 | MC_1;   // SMCLK, contmode = count up to CCR0
    TACTL |= TACLR;          // clear timer Resets TAR, clock divider, count direction

    __bis_SR_register(LPM0_bits + GIE);       // Timer interrupts with LPM0 (Sleep until the next interrupt)

// It should never get here
    while (1)           // main loop
    {
       P1OUT ^= BIT0;   // error
    }  // end while main loop

}  // end main
//==========================================================
// Timer A0 interrupt service routine
#if defined(__TI_COMPILER_VERSION__) || defined(__IAR_SYSTEMS_ICC__)
#pragma vector=TIMER0_A0_VECTOR
__interrupt void Timer_A (void)
#elif defined(__GNUC__)
void __attribute__ ((interrupt(TIMER0_A0_VECTOR))) Timer_A (void)
#else
#error Compiler not supported!
#endif
{
   asm(" nop");    // for debugger

   if(irq_count == 2)
   {
      lcd_display();
   }

   if(irq_count-- == 0)
   {
      temp_C = get_htu_temperature();   // total time to read and convert the temperature is 96ms
      F = (temp_C * 9 / 5) +320;    // Fahrenheit * 10
      Humidity = get_htu_humidity();
      irq_count = 3;
   }

   asm(" nop");    // for debugger
} // end irq
//==========================================================
void waitms(unsigned int k)     // ms
{
   volatile unsigned int i,j;

   for(i=k; i>0; i--)
   {
      for(j=99; j>0; j--);
   }

   return;
}

//==========================================================
